@@ -1,39 +1,90 @@
-# $Id: WithCache.pm 31342 2009-03-18 12:01:10Z daisuke $
 
 package MooseX::WithCache;
-use Moose;
-use Moose::Exporter;
+use MooseX::Role::Parameterized;
 use 5.008;
-our $VERSION   = '0.00005';
+use constant DEBUG => $ENV{MOOSEX_WITHCACHE_DEBUG} ? 1 : 0;
+our $VERSION   = '0.01000';
 our $AUTHORITY = 'cpan:DMAKI';
-
-Moose::Exporter->setup_import_methods(
-    with_caller => [ 'with_cache' ]
-);
-
-no Moose;
-no Moose::Exporter;
-
 my %BACKENDS;
 
+# This is solely for backwards compatibility
+use Moose::Exporter;
+Moose::Exporter->setup_import_methods(
+    with_caler => [ 'with_cache' ],
+);
+
 sub with_cache {
     my ($caller, $name, %args) = @_;
 
-    # backend is the actual cache backend. this is the main guy that
-    # does the cache-aware  meta protocol munging.
-    # we
-    my $backend_class = $args{backend} || 'Cache::Memcached';
-    my $module = "MooseX::WithCache::Backend::$backend_class";
+    Carp::carp("use of with_cache for MooseX::WithCache is now deprecated. Use parameterized roles directly");
 
-    my $backend = $BACKENDS{ $module };
-    if (! $backend) {
-        Class::MOP::load_class($module);
-        $backend = $module->new();
-        $BACKENDS{ $module } = $backend;
-    }
-    $backend->setup({ %args, cache_name => $name, package => $caller });
+    Moose::Util::apply_all_roles(
+        $caller,
+        __PACKAGE__, {
+            %args,
+            name => $name,
+        }
+    );
 }
 
+parameter backend => (
+    isa => 'Str',
+    required => 1,
+    default => 'Cache::Memcached',
+);
+
+parameter name => (
+    isa => 'Str',
+    required => 1,
+    default => 'cache'
+);
+
+role {
+    my $p = shift;
+
+    my $name          = $p->name;
+    my $backend_class = $p->backend;
+
+    if ($backend_class !~ s/^\+//) {
+        $backend_class = "MooseX::WithCache::Backend::$backend_class";
+    }
+    Class::MOP::load_class($backend_class);
+    my $backend = $BACKENDS{ $backend_class };
+    if (! $backend ) {
+        $backend = $backend_class->new();
+        $BACKENDS{ $backend_class } = $backend;
+    }
+
+    has $name => (
+        is => 'rw',
+        isa => $backend->cache_type(),
+        coerce => $backend->can_coerce(),
+    );
+
+    has cache_disabled => (
+        is => 'rw',
+        isa => 'Bool',
+        default => 0
+    );
+
+    # key generator generates the appropriate cache key from given key(s). 
+    has cache_key_generator => (
+        is      => 'rw',
+        does    => 'MooseX::WithCache::KeyGenerator',
+    );
+
+    method __get_cache => sub { $_[0]->$name };
+    method cache_debug => sub {
+        my $self = shift;
+        print STDERR "[CACHE]: @_\n";
+    };
+
+    my $methods = $backend->methods();
+    while (my($method, $code) = each %$methods) {
+        method $method, $code;
+    }
+};
+
 1;
 
 __END__
@@ -48,7 +99,7 @@ MooseX::WithCache - Easy Cache Access From Moose Objects
     use Moose;
     use MooseX::WithCache;
 
-    with_cache 'cache' => (
+    with 'MooseX::WithCache' => {
         backend => 'Cache::Memcached',
     );
 
@@ -72,7 +123,13 @@ MooseX::WithCache - Easy Cache Access From Moose Objects
     my $foo = $object->get_foo();
 
     # if you want to do something with the cache object,
-    # you can access it via the name you gave in with_cache
+    # you can access it via the name you gave in with staemtent
+    # 
+    # with 'MooseX::WithCache' => {
+    #    name => 'cache', # default
+    #    ....
+    # }
+
     my $cache = $object->cache;
 
 =head1 DESCRIPTION
@@ -130,12 +187,6 @@ Simply specify it in the constructor:
         cache_key_generator => MooseX::WithCache::KeyGenerator::DumpChecksum->new()
     );
 
-=head1 METHODS
-
-=head2 with_cache($name, %opts)
-
-Configures the cache for the object.
-
 =head1 AUTHOR
 
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
